from bs4 import BeautifulSoup
"""
"""

class Sauce(BeautifulSoup):
    """
    This class represents a customized version of the BeautifulSoup class for parsing HTML.

    :class: `Sauce` inherits from :class:`BeautifulSoup` and adds additional functionalities for parsing HTML.

    Usage:
    ======
    To use this class, simply instantiate it with an HTML string:

        sauce = Sauce("<html><head><title>Example</title></head><body><h1>Hello, world!</h1></body></html>")

    Methods:
    ========
    :class:`Sauce` provides the following methods:

    __init__(self, html)
    ---------------------
    Initializes a new instance of the Sauce class.

    Parameters:
        - html (str): The HTML string to parse.

    _markup_is_url(cls, markup)
    ---------------------------
    Checks whether the given markup resembles a URL.

    Parameters:
        - markup (str): The markup to check.

    _markup_resembles_filename(cls, markup)
    ---------------------------------------
    Checks whether the given markup resembles a filename.

    Parameters:
        - markup (str): The markup to check.

    Note:
    -----
    This class inherits the methods and attributes from the BeautifulSoup class, which can be found in the official BeautifulSoup documentation.

    """
    def __init__(self, html):
        super().__init__(html, 'html.parser')

    @classmethod
    def _markup_is_url(cls, markup):
        pass

    @classmethod
    def _markup_resembles_filename(cls, markup):
        pass



import ast

from django import template

from django_chili.sauce import Sauce


class ChopNode(template.Node):
    """
    Represents a custom template tag node that allows chopping and manipulating HTML tags.

    Args:
        content_node (Node): The node containing the content to be rendered.
        match (str): The CSS selector used to find the tags to be manipulated.
        target (str, optional): The target of the manipulation. Default is "tag".
        attr (str, optional): The attribute name to be modified. Required only if target is "attr".

    Attributes:
        tag (str): The name of the custom template tag.

    Methods:
        render(context): Renders the custom template tag node.

    """
    tag = "chop"

    def __init__(self, content_node, match, target="tag", attr=None):
        self.content_node = content_node
        self.match = match
        self.target = target
        self.attr = attr

    def render(self, context):
        """
        Renders the content based on the given context.

        :param context: The context containing the data.
        :type context: dict
        :return: The rendered content.
        :rtype: str
        """
        tags = context['sauce'].select(self.match)

        for tag in tags:
            context['tag'] = tag
            content = self.content_node.render(context)
            content = str(content)
            content_tag = Sauce(content)

            match self.target:
                case "tag":
                    tag.replace_with(content_tag)
                case "inner":
                    tag.clear()
                    tag.append(content_tag)
                case "attr":
                    tag.attrs.pop(self.attr)
                    tag.attrs.update({self.attr: content_tag})

        return ""


def chop_tag(parser, token):
    """
    :param parser: The parser object used for parsing the template.
    :param token: The tag's token string.
    :return: The parsed ChopNode object.

    This method takes a parser object and a token string as input parameters and returns a parsed ChopNode object.

    The method works by splitting the token string into the tag name and arguments. It then processes the arguments by splitting them further into positional arguments and named arguments
    *. Positional arguments are added to a list while named arguments are added to a dictionary.

    After processing the arguments, the method calls the parser to parse the content node using the 'endChopNode' tag as the end point. It then deletes the first token and returns a Chop
    *Node object created from the parsed content node and the positional and named arguments.
    """
    tag_name, args = token.contents.split(None, 1)

    parts = args.split(',')

    positional_arguments = []
    named_arguments = {}

    for part in parts:
        arg = part.strip().split('=')

        if len(arg) == 1:
            positional_arguments.append(ast.literal_eval(arg[0].strip()))
        elif len(arg) == 2:
            named_arguments[arg[0]] = ast.literal_eval(arg[1].strip())

    content_node = parser.parse((f'end{ChopNode.tag}',))
    parser.delete_first_token()
    return ChopNode(content_node, *positional_arguments, **named_arguments)



from django import template

from django_chili.bowl import bowl_tag, BowlNode
from django_chili.chop import chop_tag, ChopNode
from django_chili.chunk import chunk_tag

register = template.Library()

register.tag(compile_function=bowl_tag, name=BowlNode.tag)
register.tag(compile_function=chop_tag, name=ChopNode.tag)
register.simple_tag(func=chunk_tag, takes_context=True, name="chunk", )



from django.utils.safestring import mark_safe


def chunk_tag(context):
    """
    Process the given context and return the HTML tag as a safe string.

    :param context: A dictionary containing the context data.
    :type context: dict

    :return: The HTML tag as a safe string.
    :rtype: safe string
    """
    tag = context['tag']
    html = str(tag)
    return mark_safe(html)



from django import template
from django.utils.safestring import mark_safe

from django_chili.sauce import Sauce


class BowlNode(template.Node):
    """

    This class represents a custom template node for rendering a Bowl.

        - `BowlNode` is a subclass of `template.Node`.

    Attributes:
        - `tag` (str): Represents the tag name used in the template for this node.

    Methods:
        - `__init__(self, content_node: template.Node, filename: str)`:
            Initializes a new `BowlNode` instance with the given `content_node` and `filename`. The `content_node` represents the content of the node in the template, and the `filename`
    * is a string representing the path to the file containing the contents of the bowl.

        - `render(self, context: template.Context) -> str`:
            Renders the `BowlNode` by reading the contents of the file specified by `filename` and creating a `Sauce` instance with the contents. The `Sauce` instance is then added to the
    * context as a variable named 'sauce'. The `content_node` is then rendered in the provided context. Finally, the `str` representation of the `Sauce` instance is returned as HTML.

    """
    tag = 'bowl'

    def __init__(self, content_node, filename):
        self.content_node = content_node

        with open(filename, 'r') as file:
            self.contents = file.read()

    def render(self, context):
        """
        Render Method

        This method is used to render the contents of a Sauce instance with the provided context.

        :param context: The context dictionary containing the values to be rendered.

        :return: The rendered HTML as a string.

        """
        sauce = Sauce(self.contents)
        context['sauce'] = sauce
        self.content_node.render(context)
        html = str(sauce)
        return mark_safe(html)


def bowl_tag(parser, token):
    """
    :param parser: The Django template parser object.
    :param token: The Django template token object.
    :return: A BowlNode object.

    This method is a custom template tag for the Django templating engine. It takes two parameters, `parser` which is the template parser object, and `token` which is the template token
    * object. It performs a specific logic to parse the tag name and filename from the token contents, then parses the content nodes using the parser, deletes the first token, and returns
    * a BowlNode object with the parsed content node and stripped filename.
    """
    tag_name, filename = token.contents.split(None, 1)
    content_node = parser.parse((f'end{BowlNode.tag}',))
    parser.delete_first_token()
    return BowlNode(content_node, filename.strip())
